@@ -26,7 +26,6 @@ use cargo_metadata::{Artifact, CrateType, Message, Metadata, MetadataCommand, Pa
 use semver::Version;
 use shell_escape::escape;
 use tempfile::NamedTempFile;
-use wasm_metadata::{Link, LinkType, RegistryMetadata};
 use wasm_pkg_client::{
     caching::{CachingClient, FileCache},
     PackageRef, PublishOpts, Registry,
@@ -971,14 +970,8 @@ fn componentize(
         })?
         .validate(true);
 
-    let mut producers = wasm_metadata::Producers::empty();
-    producers.add(
-        "processed-by",
-        env!("CARGO_PKG_NAME"),
-        option_env!("CARGO_VERSION_INFO").unwrap_or(env!("CARGO_PKG_VERSION")),
-    );
-
-    let component = producers.add_to_wasm(&encoder.encode()?).with_context(|| {
+    let package = &cargo_metadata[&artifact.package_id];
+    let component = add_component_metadata(&package, &encoder.encode()?).with_context(|| {
         format!(
             "failed to add metadata to output component `{path}`",
             path = path.display()
@@ -1027,57 +1020,46 @@ pub struct PublishOptions<'a> {
     pub dry_run: bool,
 }
 
-fn add_registry_metadata(package: &Package, bytes: &[u8], path: &Path) -> Result<Vec<u8>> {
-    let mut metadata = RegistryMetadata::default();
-    if !package.authors.is_empty() {
-        metadata.set_authors(Some(package.authors.clone()));
-    }
-
-    if !package.categories.is_empty() {
-        metadata.set_categories(Some(package.categories.clone()));
-    }
-
-    metadata.set_description(package.description.clone());
-
-    // TODO: registry metadata should have keywords
-    // if !package.keywords.is_empty() {
-    //     metadata.set_keywords(Some(package.keywords.clone()));
-    // }
-
-    metadata.set_license(package.license.clone());
-
-    let mut links = Vec::new();
-    if let Some(docs) = &package.documentation {
-        links.push(Link {
-            ty: LinkType::Documentation,
-            value: docs.clone(),
-        });
-    }
-
-    if let Some(homepage) = &package.homepage {
-        links.push(Link {
-            ty: LinkType::Homepage,
-            value: homepage.clone(),
-        });
-    }
-
-    if let Some(repo) = &package.repository {
-        links.push(Link {
-            ty: LinkType::Repository,
-            value: repo.clone(),
-        });
-    }
-
-    if !links.is_empty() {
-        metadata.set_links(Some(links));
-    }
-
-    metadata.add_to_wasm(bytes).with_context(|| {
-        format!(
-            "failed to add registry metadata to component `{path}`",
-            path = path.display()
-        )
-    })
+/// Read metadata from `Cargo.toml` and add it to the component
+fn add_component_metadata(package: &Package, wasm: &[u8]) -> Result<Vec<u8>> {
+    let metadata = wasm_metadata::AddMetadata {
+        name: Some(package.name.clone()),
+        language: vec![("Rust".to_string(), "".to_string())],
+        processed_by: vec![(
+            env!("CARGO_PKG_NAME").to_string(),
+            option_env!("CARGO_VERSION_INFO")
+                .unwrap_or(env!("CARGO_PKG_VERSION"))
+                .to_string(),
+        )],
+        sdk: vec![],
+        authors: match package.authors.len() {
+            0 => None,
+            _ => Some(wasm_metadata::Authors::new(package.authors.join(","))),
+        },
+        description: package
+            .description
+            .as_ref()
+            .map(|d| wasm_metadata::Description::new(d.clone())),
+        licenses: package
+            .license
+            .as_ref()
+            .map(|s| wasm_metadata::Licenses::new(&s))
+            .transpose()?,
+        source: package
+            .repository
+            .as_ref()
+            .map(|s| wasm_metadata::Source::new(s.to_string().as_str()))
+            .transpose()?,
+        homepage: package
+            .homepage
+            .as_ref()
+            .map(|s| wasm_metadata::Homepage::new(s.to_string().as_str()))
+            .transpose()?,
+        // TODO: get the git commit hash
+        revision: None,
+        version: Some(wasm_metadata::Version::new(package.version.to_string())),
+    };
+    metadata.to_wasm(wasm)
 }
 
 /// Publish a component for the given workspace and publish options.
@@ -1100,8 +1082,6 @@ pub async fn publish(
         )
     })?;
 
-    let bytes = add_registry_metadata(options.package, &bytes, options.path)?;
-
     config.terminal().status(
         "Publishing",
         format!("component {path}", path = options.path.display()),
