@@ -71,7 +71,7 @@ fn it_builds_a_workspace() -> Result<()> {
         r#"[package]
 name = "baz"
 version = "0.1.0"
-edition = "2021"
+edition = "2024"
 
 [dependencies]
 "#,
@@ -128,36 +128,6 @@ fn it_supports_wit_keywords() -> Result<()> {
     Ok(())
 }
 
-#[test]
-fn it_adds_a_producers_field() -> Result<()> {
-    let project = Project::new("foo", true)?;
-
-    project
-        .cargo_component(["build", "--release"])
-        .assert()
-        .stderr(contains("Finished `release` profile [optimized] target(s)"))
-        .success();
-
-    let path = project.release_wasm("foo");
-
-    validate_component(&path)?;
-
-    let wasm = fs::read(&path)
-        .with_context(|| format!("failed to read wasm file `{path}`", path = path.display()))?;
-    let section = wasm_metadata::Producers::from_wasm(&wasm)?.expect("missing producers section");
-
-    assert_eq!(
-        section
-            .get("processed-by")
-            .expect("missing processed-by field")
-            .get(env!("CARGO_PKG_NAME"))
-            .expect("missing cargo-component field"),
-        option_env!("CARGO_VERSION_INFO").unwrap_or(env!("CARGO_PKG_VERSION"))
-    );
-
-    Ok(())
-}
-
 #[test]
 fn it_builds_wasm32_unknown_unknown_from_cli() -> Result<()> {
     let project = Project::new("foo", true)?;
@@ -421,7 +391,7 @@ fn empty_world_with_dep_valid() -> Result<()> {
             #[allow(warnings)]
             mod bindings;
 
-            #[no_mangle]
+            #[unsafe(no_mangle)]
             pub extern \"C\" fn foo() {
                 bindings::foo_bar::hello();
             }
@@ -955,3 +925,185 @@ fn it_does_not_generate_bindings_for_cargo_projects() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+/// This is exactly the `it_builds_a_workspace` test with just the edition changed to 2021.
+fn it_supports_edition_2021() -> Result<()> {
+    let dir = Rc::new(TempDir::new()?);
+    let project = Project::new_uninitialized(dir.clone(), dir.path().to_owned());
+
+    project.file(
+        "baz/Cargo.toml",
+        r#"[package]
+name = "baz"
+version = "0.1.0"
+edition = "2021"
+
+[dependencies]
+"#,
+    )?;
+
+    project.file("baz/src/lib.rs", "")?;
+
+    project
+        .cargo_component(["new", "--lib", "foo"])
+        .assert()
+        .stderr(contains("Updated manifest of package `foo`"))
+        .success();
+
+    project
+        .cargo_component(["new", "--lib", "bar"])
+        .assert()
+        .stderr(contains("Updated manifest of package `bar`"))
+        .success();
+
+    // Add the workspace after all of the projects have been created
+    project.file(
+        "Cargo.toml",
+        r#"[workspace]
+    members = ["foo", "bar", "baz"]
+    "#,
+    )?;
+
+    project
+        .cargo_component(["build"])
+        .assert()
+        .stderr(contains(
+            "Finished `dev` profile [unoptimized + debuginfo] target(s)",
+        ))
+        .success();
+
+    validate_component(&project.debug_wasm("foo"))?;
+    validate_component(&project.debug_wasm("bar"))?;
+
+    Ok(())
+}
+
+#[test]
+fn it_adds_a_producers_field() -> Result<()> {
+    let project = Project::new("foo", true)?;
+
+    project
+        .cargo_component(["build", "--release"])
+        .assert()
+        .stderr(contains("Finished `release` profile [optimized] target(s)"))
+        .success();
+
+    let path = project.release_wasm("foo");
+
+    validate_component(&path)?;
+
+    let wasm = fs::read(&path)
+        .with_context(|| format!("failed to read wasm file `{path}`", path = path.display()))?;
+    let section = wasm_metadata::Producers::from_wasm(&wasm)?.expect("missing producers section");
+
+    assert_eq!(
+        section
+            .get("processed-by")
+            .expect("missing processed-by field")
+            .get(env!("CARGO_PKG_NAME"))
+            .expect("missing cargo-component field"),
+        option_env!("CARGO_VERSION_INFO").unwrap_or(env!("CARGO_PKG_VERSION"))
+    );
+
+    Ok(())
+}
+
+#[test]
+fn it_adds_metadata_from_cargo_toml() -> Result<()> {
+    let name = "foo";
+    let authors = "Jane Doe <jane@example.com>";
+    let description = "A test package";
+    let license = "Apache-2.0";
+    let version = "1.0.0";
+    let documentation = "https://example.com/docs";
+    let homepage = "https://example.com/home";
+    let repository = "https://example.com/repo";
+
+    let project = Project::new(name, true)?;
+    project.update_manifest(|mut doc| {
+        let package = &mut doc["package"];
+        package["name"] = value(name);
+        package["version"] = value(version);
+        package["authors"] = value(Array::from_iter([authors]));
+        package["description"] = value(description);
+        package["license"] = value(license);
+        package["documentation"] = value(documentation);
+        package["homepage"] = value(homepage);
+        package["repository"] = value(repository);
+        Ok(doc)
+    })?;
+
+    project
+        .cargo_component(["build", "--release"])
+        .assert()
+        .stderr(contains("Finished `release` profile [optimized] target(s)"))
+        .success();
+
+    let path = project.release_wasm("foo");
+
+    validate_component(&path)?;
+
+    let wasm = fs::read(&path)
+        .with_context(|| format!("failed to read wasm file `{path}`", path = path.display()))?;
+
+    let metadata = match wasm_metadata::Payload::from_binary(&wasm)? {
+        wasm_metadata::Payload::Component { metadata, .. } => metadata,
+        wasm_metadata::Payload::Module(_) => unreachable!("found a wasm module"),
+    };
+
+    assert_eq!(
+        &metadata.name.as_ref().expect("missing name").to_string(),
+        name
+    );
+    assert_eq!(
+        &metadata
+            .authors
+            .as_ref()
+            .expect("missing authors")
+            .to_string(),
+        authors
+    );
+    assert_eq!(
+        &metadata
+            .description
+            .as_ref()
+            .expect("missing description")
+            .to_string(),
+        description
+    );
+    assert_eq!(
+        &metadata
+            .licenses
+            .as_ref()
+            .expect("missing licenses")
+            .to_string(),
+        license
+    );
+    assert_eq!(
+        &metadata
+            .source
+            .as_ref()
+            .expect("missing source")
+            .to_string(),
+        repository
+    );
+    assert_eq!(
+        &metadata
+            .homepage
+            .as_ref()
+            .expect("missing homepage")
+            .to_string(),
+        homepage
+    );
+    assert_eq!(
+        &metadata
+            .version
+            .as_ref()
+            .expect("missing version")
+            .to_string(),
+        version
+    );
+
+    Ok(())
+}
